@@ -17,19 +17,23 @@ typedef struct cm_cell {
 	long cellID;			/* cell ID */
 	struct cm_cell *nextp;		/* locked by cm_cellLock */
         char *namep;			/* cell name; never changes */
-        struct cm_serverRef *vlServersp;	/* locked by cm_serverLock */
+        cm_serverRef_t *vlServersp;     /* locked by cm_serverLock */
         osi_mutex_t mx;			/* mutex locking fields (flags) */
         long flags;			/* locked by mx */
         long timeout;                   /* if dns, time at which the server addrs expire */
 } cm_cell_t;
 
-#define CM_CELLFLAG_SUID	1	/* setuid flag; not yet used */
-#define CM_CELLFLAG_DNS         2       /* cell servers are from DNS */
+/* These are bit flag values */
+#define CM_CELLFLAG_SUID	    1	/* setuid flag; not yet used */
+#define CM_CELLFLAG_DNS         2   /* cell servers are from DNS */
+#define CM_CELLFLAG_VLSERVER_INVALID 4  /* cell servers are invalid */
 
 extern void cm_InitCell(void);
 
 extern cm_cell_t *cm_GetCell(char *namep, long flags);
 
+extern cm_cell_t *cm_GetCell_Gen(char *namep, char *newnamep, long flags);
+
 extern cm_cell_t *cm_FindCellByID(long cellID);
 
 extern void cm_ChangeRankCellVLServer(cm_server_t       *tsp);
